﻿using System;
using System.Windows;
using System.Windows.Controls;
using Ozeki.Media;
using Ozeki.Media.MediaHandlers;
using Ozeki.VoIP;
using Ozeki.VoIP.Media;
using Ozeki.VoIP.SDK;
using Ozeki.Media.DSP;

namespace SilenceFilter
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        ISoftPhone softPhone;
        IPhoneCall phoneCall;
        IPhoneLine phoneLine;
        VADFilter vadFilter;
        Microphone microphone;
        Speaker speaker;
        MediaConnector mediaConnector;
        PhoneCallAudioSender phoneCallAudioSender;
        PhoneCallAudioReceiver phoneCallAudioReceiver;
        AudioQualityEnhancer audioProcessor;

        public MainWindow()
        {
            Loaded += MainWindow_Loaded;
            InitializeComponent();
        }

        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            InitSoftPhone();
            InitMediaHandlers();
        }

        void InitSoftPhone()
        {
            var sipAccount = new SIPAccount(true, "oz890", "oz890", "oz890", "oz890", "192.168.112.100");
            txtBlockAccount.Text = "oz890";

            softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5060, 5080, 5060);
            phoneLine = softPhone.CreatePhoneLine(sipAccount);
            phoneLine.PhoneLineStateChanged += phoneLine_PhoneLineStateChanged;

            softPhone.RegisterPhoneLine(phoneLine);

        }

        void InitMediaHandlers()
        {
            mediaConnector = new MediaConnector();
            audioProcessor = new AudioQualityEnhancer();

            microphone = Microphone.GetDefaultDevice();
            speaker = Speaker.GetDefaultDevice();

            vadFilter = new VADFilter();
            phoneCallAudioReceiver = new PhoneCallAudioReceiver();
            phoneCallAudioSender = new PhoneCallAudioSender();

            mediaConnector.Connect(microphone, audioProcessor);
            mediaConnector.Connect(audioProcessor, vadFilter);
            mediaConnector.Connect(vadFilter, phoneCallAudioSender);
            mediaConnector.Connect(phoneCallAudioReceiver, speaker);

            speaker.Start();
            microphone.Start();
        }


        void phoneLine_PhoneLineStateChanged(object sender, VoIPEventArgs<PhoneLineState> e)
        {
            InvokeOnGUIThread(() =>
            {
                txtBlockStatus.Text = e.Item.ToString();
            });

        }

        void Btn_Click(object sender, RoutedEventArgs e)
        {
            var btn = sender as Button;

            if (btn != null)
                txtBoxDisplay.Text += btn.Tag;
        }

        void Btn_PickUp(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrEmpty(txtBoxDisplay.Text))
                return;

            if (phoneCall != null)
                return;

            if (phoneLine.RegisteredInfo != PhoneLineState.RegistrationSucceeded && phoneLine.RegisteredInfo != PhoneLineState.NoRegNeeded)
                return;

            phoneCall = softPhone.CreateCallObject(phoneLine, txtBoxDisplay.Text, CallType.Audio);
            phoneCall.CallStateChanged += phoneCall_CallStateChanged;
            phoneCall.CallErrorOccured += phoneCall_CallErrorOccured;

            phoneCall.Start();
        }

        void phoneCall_CallErrorOccured(object sender, VoIPEventArgs<CallError> e)
        {
            InvokeOnGUIThread(() => MessageBox.Show("Call error occurred: " + e.Item));

        }

        void phoneCall_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            if (e.Item.IsInCall())
            {
                phoneCallAudioReceiver.AttachToCall(phoneCall);
                phoneCallAudioSender.AttachToCall(phoneCall);

            }
            else if (e.Item.IsCallEnded())
            {
                phoneCallAudioReceiver.Detach();
                phoneCallAudioSender.Detach();

                phoneCall = null;

                InvokeOnGUIThread(() =>
                {
                    txtBoxDisplay.Text = string.Empty;
                });

            }
        }

        void Btn_HangUP(object sender, RoutedEventArgs e)
        {
            txtBoxDisplay.Text = string.Empty;

            if (phoneCall == null)
                return;

            phoneCall.HangUp();
            phoneCall = null;


        }

        void InvokeOnGUIThread(Action action)
        {
            if (Dispatcher.CheckAccess())
                action();
            else
                Dispatcher.BeginInvoke(action);
        }
    }
}
